﻿using System;
using System.IO;

namespace WpfOsi.Helper.IO
{
    public class XmlSerializer
    {
        /// <summary>
        /// Der XMLSerializer serialisiert eine beliebige Klasse welche auch Primitiven Datentypen oder solchen welche als Serialisiert gekennzeichnet sind
        /// Klasse ist generisch aufgebaut und gibt die selbe Type von Klasse zurück welche ihm übergeben wird.
        /// </summary>
        /// 
        public void Serialize<T>(string path, T instance)
        {
            try
            {
                string dirName = Path.GetDirectoryName(path);
                if (!Directory.Exists(dirName))
                    Directory.CreateDirectory(dirName);

                File.Delete(path);

                using (FileStream fs = new FileStream(path, FileMode.OpenOrCreate))
                {
                    SaveToStream(fs, instance);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message, ex.InnerException);
            }
        }
        /// <summary>
        /// Serialisiert eine Klasse über einen Stream
        /// </summary>
        /// <typeparam name="T">Den Klassentyp angeben</typeparam>
        /// <param name="s">Der Stream welcher die Daten enthält (Filestream, MemoryStream,...)</param>
        /// <param name="o">Die Instanz der Klasse welche serialisiert werden soll</param>
        public void SaveToStream<T>(Stream s, T o)
        {
            try
            {
                System.Xml.Serialization.XmlSerializer x = new System.Xml.Serialization.XmlSerializer(typeof(T));
                x.Serialize(s, o);
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message, ex.InnerException);
            }
        }
        
        /// <summary>
        /// Deserialisiert eine XML in eine Klasseninstanz - Die Instanz muss also nicht erstellt sein.
        /// </summary>
        /// <typeparam name="T">Der Typ der Klasse welche erwartet wird.</typeparam>
        /// <param name="path">Der Pfad zur XML-Datei inkl. Dateiendung</param>
        /// <param name="defaultInstance">Die Instanz der Klasse falls die Datei noch nicht Existiert oder nicht gefunden werden kann</param>
        /// <returns>Gibt die Deserialissierte Klasse zurück</returns>
        public T DeSerialize<T>(string path, T defaultInstance)
        {
            try
            {
                if(!File.Exists(path))
                {
                    return defaultInstance;
                }
                using (FileStream fs = new FileStream(path,FileMode.OpenOrCreate))
                {
                    return LoadFromStream<T>(fs);
                }
            }
            catch (Exception ex)
            {

                throw new Exception(ex.Message, ex.InnerException);
            }
        }
        /// <summary>
        /// Läd ein Klasse generisch über einen Stream, Methode ist gut geeignet für UnitTests
        /// </summary>
        /// <typeparam name="T">Der Typ der Klasse welche erwartet wird</typeparam>
        /// <param name="s">Der Stream welcher die Daten enthält (Filestream, MemoryStream,...)</param>
        /// <returns>Gibt die Deserialisierte Klasse zurück</returns>
        public T LoadFromStream<T>(Stream s)
        {
            System.Xml.Serialization.XmlSerializer x = new System.Xml.Serialization.XmlSerializer(typeof(T));
            return (T)x.Deserialize(s);
        }
    }
}
