﻿using System;
using System.Windows.Input;
using WpfOsi.BaseClasses;
using WpfOsi.Helper;

namespace WpfOsi.Workspace
{
    public class HeaderWorkspace:ViewModelBase
    {
        internal readonly MainWorkspace _mainWs;

        public HeaderWorkspace()
        {
            if (!IsInDesignMode)
                throw new Exception("Dieser Konstruktor ist nur für den DesignTime-Support!");
        }
        public HeaderWorkspace(MainWorkspace mainWs)
        {
            _mainWs = mainWs ?? throw new ArgumentNullException(nameof(mainWs));
        }
        public string ModeButtonText
        {
            get
            {
                if (_mainWs == null) return "Edit Mode not possible";
                return _mainWs.IsInEditMode ? "Save and stop editing" : "Switch on edit mode";
            }
        }
        #region Commands
        private ICommand _switchEditableCommand;
        public ICommand SwitchEditableCommand
        {
            get { return _switchEditableCommand ?? new RelayCommand(SwitchEditableCommand_Execute, SwitchEditableCommand_CanExecute); }
            set{ _switchEditableCommand = value; RaisePropertyChanged();}
        }
        private bool SwitchEditableCommand_CanExecute(object obj)
        {
            return _mainWs != null;
        }
        private void SwitchEditableCommand_Execute(object obj)
        {
            _mainWs.IsInEditMode = !_mainWs.IsInEditMode;
            RaisePropertyChanged(nameof(ModeButtonText));
        }

        private ICommand _manageProductionLineCommand;
        public ICommand ManageProductionLineCommand
        {
            get {return _manageProductionLineCommand ?? new RelayCommand(ManageProductionLineCommand_Execute, ManageProductionLineCommand_CanExecute);}
            set { _manageProductionLineCommand = value; RaisePropertyChanged(); }
        }
        private bool ManageProductionLineCommand_CanExecute(object obj)
        {
            return _mainWs != null && _mainWs.IsInEditMode;
        }
        private void ManageProductionLineCommand_Execute(object obj)
        {
            //ShowProductionLineWindow plWindow = new ShowProductionLineWindow();
            _mainWs.Ctx.SaveChanges();
            //plWindow.ShowDialog();
            if (RefreshDataCommand.CanExecute(null)) RefreshDataCommand.Execute(null);
        }
        private ICommand _manageInstructionCommand;
        public ICommand ManageInstructionCommand
        {
            get { return _manageInstructionCommand ?? new RelayCommand(ManageInstructionCommand_Execute, ManageInstructionCommand_CanExecute); }
            set { _manageInstructionCommand = value; RaisePropertyChanged(); }
        }
        private bool ManageInstructionCommand_CanExecute(object obj)
        {
            return _mainWs != null && _mainWs.IsInEditMode;
        }
        private void ManageInstructionCommand_Execute(object obj)
        {
            //ShowInstructionWindow instWindow = new ShowInstructionWindow();
            _mainWs.Ctx.SaveChanges();
            //instWindow.ShowDialog();
            if (RefreshDataCommand.CanExecute(null)) RefreshDataCommand.Execute(null);
        }
        private ICommand _refreshDataCommand;
        public ICommand RefreshDataCommand
        {
            get { return _refreshDataCommand ?? new RelayCommand(RefreshDataCommand_Execute, RefreshDataCommand_CanExecute); }
            set { _refreshDataCommand = value; RaisePropertyChanged(); }
        }
        private bool RefreshDataCommand_CanExecute(object obj)
        {
            return true;
        }
        private void RefreshDataCommand_Execute(object obj)
        {
            _mainWs.Ctx.LoadContext();
            _mainWs.OsiWs.ReloadData();
            _mainWs.StatusWs.LastMessage = "";
        }
        #endregion
    }

}
