﻿using System.Windows;
using WpfOsi.BaseClasses;
using WpfOsi.Context;
namespace WpfOsi.Workspace
{
    public class MainWorkspace:ViewModelBase
    {
        internal readonly OsiContext Ctx;

        public MainWorkspace()
        {
            if (IsInDesignMode) { }
            else
            {
                Ctx = new OsiContext();
                HeaderWs = new HeaderWorkspace(this);
                OsiWs = new OSIWorkspace(this);
                StatusWs = new StatusWorkspace();
                Ctx.DataFileWasChanged += DataFileChanged;
            }
        }
        private void DataFileChanged()
        {
            StatusWs.LastMessage = "The data file has been edited elsewhere. You should reload the data.";
        }
        #region Workspaces
        private HeaderWorkspace _headerWs;
        public HeaderWorkspace HeaderWs
        {
            get { return _headerWs; }
            set { _headerWs = value; RaisePropertyChanged(); }
        }
        private StatusWorkspace _statusWs;

        public StatusWorkspace StatusWs
        {
            get { return _statusWs; }
            set { _statusWs = value; RaisePropertyChanged(); }
        }
        private OSIWorkspace _osiWs;

        public OSIWorkspace OsiWs
        {
            get { return _osiWs; }
            set { _osiWs = value;RaisePropertyChanged(); }
        }
        #endregion
        private bool _isInEditMode;
        public bool IsInEditMode
        {
            get { return _isInEditMode; }
            set
            {
                if (_isInEditMode) GoToReadOnlyMode();
                else
                {
                    if (IsLockedForChanges)
                    {
                        MessageBox.Show("Unfortunately, the file is currently locked. Try again later.");
                        return;
                    }
                    if (!Ctx.IsDataUpToDate)
                    {
                        MessageBox.Show("The data has been edited by someone else. Please update first and then set to the edit mode.");
                        return;
                    }
                    GoToEditMode();
                }
                _isInEditMode = value;
                RaisePropertyChanged();
            }
        }
        public bool IsLockedForChanges { get { return Ctx == null || Ctx.HasPessimisticLock; } }
        private void GoToEditMode()
        {
            Ctx.LockForChanges();
            RaisePropertyChanged(nameof(IsLockedForChanges));
        }
        private void GoToReadOnlyMode()
        {
            Ctx.UnlockForChanges();
            Ctx.SaveChanges();
            RaisePropertyChanged(nameof(IsLockedForChanges));
        }
    }
}
