﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Data;
using System.Windows.Input;
using WpfOsi.BaseClasses;
using WpfOsi.Helper;
using WpfOsi.Model;

namespace WpfOsi.Workspace
{
    public class OSIWorkspace:ViewModelBase
    {
        public MainWorkspace MainWs { get; set; }
        public OSIWorkspace()
        {
            if (!IsInDesignMode) throw new Exception("Dieser Konstruktor ist nur für den DesignTime-Support!");
            else
            { 
               FillAvailables();
            }
        }
        public OSIWorkspace(MainWorkspace mainws)
        {
            MainWs = mainws ?? throw new ArgumentNullException(nameof(mainws));
            ReloadData();
        }
        #region Internal Voids
        internal void ReloadData()
        {
            AvailableProductionLines = new ObservableCollection<ProductionLine>(MainWs.Ctx.ProductionLines);
            AvailableInstructions = new ObservableCollection<Instruction>(MainWs.Ctx.Instructions);
            AvailableMainSteps = new ObservableCollection<MainStep>(MainWs.Ctx.MainSteps);
            AvailableChildSteps = new ObservableCollection<ChildStep>(MainWs.Ctx.ChildSteps);

            SelectedFilterProductionLine = AvailableProductionLineFilter.First();

            AvailableInstructionView = CollectionViewSource.GetDefaultView(AvailableInstructions);
            AvailableInstructionView.Filter = new Predicate<object>(InstructionFilter);

            AvailableMainStepView = CollectionViewSource.GetDefaultView(AvailableMainSteps);

            AvailableChildStepView = CollectionViewSource.GetDefaultView(AvailableChildSteps);
        }
        internal void FillAvailables()
        {
            AvailableProductionLines = new ObservableCollection<ProductionLine>()
                {
                new ProductionLine() {Designation = "BioCel"},
                new ProductionLine() {Designation = "Flachmembran" }
                };
            AvailableInstructions = new ObservableCollection<Instruction>()
                {
                new Instruction() {FormularNumber = 1000,FormularName = "TestFormular",RevisionNumber = 1,RevisionDate = DateTime.Now,LineID = AvailableProductionLines.First().ID }
                };
            AvailableMainSteps = new ObservableCollection<MainStep>()
                {
                    new MainStep() { StepNo = 1,StepText = "1. Main Step",StepImage = null,ImageText = "Image Text von Main Step 1",InstructionID = AvailableInstructions.First().ID }
                };
            AvailableChildSteps = new ObservableCollection<ChildStep>()
                {
                    new ChildStep() { StepNo = 1, StepText = "1. Child Step", StepImage = null, ImageText = "Image Text von Child Step 1", MainStepID = AvailableMainSteps.First().ID }
                };
            AvailableInstructionView = CollectionViewSource.GetDefaultView(AvailableInstructions);
            AvailableMainStepView = CollectionViewSource.GetDefaultView(AvailableMainSteps);
            AvailableChildStepView = CollectionViewSource.GetDefaultView(AvailableChildSteps);
        }
        #endregion
        #region "Views"
        private ICollectionView _availableInstructionView;

        public ICollectionView AvailableInstructionView
        {
            get { return _availableInstructionView; }
            set { _availableInstructionView = value; RaisePropertyChanged(); }
        }
        private ICollectionView _availableMainStepView;

        public ICollectionView AvailableMainStepView
        {
            get { return _availableMainStepView; }
            set { _availableMainStepView = value; RaisePropertyChanged(); }
        }
        private ICollectionView _availableChildStepView;

        public ICollectionView AvailableChildStepView
        {
            get { return _availableChildStepView; }
            set { _availableChildStepView = value; RaisePropertyChanged(); }
        }
        #endregion
        #region "Collections"
        private ObservableCollection<ProductionLine> _availableProductionLines;
        public ObservableCollection<ProductionLine> AvailableProductionLines
        {
            get { return _availableProductionLines; }
            set { _availableProductionLines = value; RaisePropertyChanged(); }
        }
        private ObservableCollection<Instruction> _availableInstructions;
        internal ObservableCollection<Instruction> AvailableInstructions
        {
            get { return _availableInstructions; }
            set { _availableInstructions = value; RaisePropertyChanged(); }
        }
        private ObservableCollection<MainStep> _availableMainSteps;
        internal ObservableCollection<MainStep> AvailableMainSteps
        {
            get { return _availableMainSteps; }
            set { _availableMainSteps = value; RaisePropertyChanged(); }
        }
        private ObservableCollection<ChildStep> _availableChildSteps;
        internal ObservableCollection<ChildStep> AvailableChildSteps
        {
            get { return _availableChildSteps; }
            set { _availableChildSteps = value; RaisePropertyChanged(); }
        }
        #endregion
        #region "Filter"
        private bool InstructionFilter(object obj)
        {
            var instruction = (Instruction)obj;
            if (SelectedFilterProductionLine != null && SelectedFilterProductionLine.ID != Guid.Empty && ((Instruction)obj).LineID.ToString() != SelectedFilterProductionLine.ID.ToString())
                return false;
            if (!String.IsNullOrEmpty(FilterString) && !instruction.FormularName.ToLower().Contains(FilterString.ToLower()))
                return false;
            return true;
        }
        private ObservableCollection<ProductionLine> _availableProductionLineFilter;

        public ObservableCollection<ProductionLine> AvailableProductionLineFilter
        {
            get
            {
                _availableProductionLineFilter = new ObservableCollection<ProductionLine>(AvailableProductionLines.ToList());
                _availableProductionLineFilter.Insert(0, new ProductionLine() { Designation = "All", ID = Guid.Empty });
                return _availableProductionLineFilter;
            }
        }


        private ProductionLine _selectedFilterProductionLine;
        public ProductionLine SelectedFilterProductionLine
        {
            get { return _selectedFilterProductionLine; }
            set
            {
                _selectedFilterProductionLine = value;
                RaisePropertyChanged();
                if (AvailableInstructionView != null)
                {
                    AvailableInstructionView.MoveCurrentToFirst();
                    AvailableInstructionView.Refresh();
                }
            }
        }
        private string _filterString;
        public string FilterString
        {
            get { return _filterString; }
            set
            {
                _filterString = value;
                RaisePropertyChanged();
                if (AvailableInstructionView != null)
                    AvailableInstructionView.Refresh();
            }
        }
        #endregion
        #region "Commands"
        private ICommand _addInstructionCommand;
        public ICommand AddInstructionCommand
        {
            get { return _addInstructionCommand ?? new RelayCommand(AddInstructionCommand_Execute, AddInstructionCommand_CanExecute); }
            set { _addInstructionCommand = value; RaisePropertyChanged(); }
        }
        private bool AddInstructionCommand_CanExecute(object obj)
        {
            return MainWs != null && MainWs.IsInEditMode;
        }
        private void AddInstructionCommand_Execute(object obj)
        {
            AvailableInstructions.Add(new Instruction() { LineID = AvailableProductionLines.First().ID });
            AvailableInstructionView.Refresh();
            AvailableInstructionView.MoveCurrentToLast();
        }
        private ICommand _removeInstructionCommand;
        public ICommand RemoveInstructionCommand
        {
            get { return _removeInstructionCommand ?? new RelayCommand(RemoveInstructionCommand_Execute, RemoveInstructionCommand_CanExecute); }
            set { _removeInstructionCommand = value; RaisePropertyChanged(); }
        }
        private bool RemoveInstructionCommand_CanExecute(object obj)
        {
            return MainWs != null && MainWs.IsInEditMode && AvailableInstructionView.CurrentItem != null;
        }
        private void RemoveInstructionCommand_Execute(object obj)
        {
            Instruction instruction = (Instruction)AvailableInstructionView.CurrentItem;
            if (MessageBox.Show("Really Delete?", "Delete", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                AvailableInstructions.Remove(instruction);
        }
        #endregion
    }
}
